"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.readFileSync = exports.readFile = void 0;
// cSpell:ignore curr
// cSpell:words zlib iconv
const fs = __importStar(require("fs"));
const Stream = __importStar(require("stream"));
const url_1 = require("url");
const util_1 = require("util");
const zlib = __importStar(require("zlib"));
const encode_decode_js_1 = require("../../common/encode-decode.js");
const transformers_js_1 = require("../../common/transformers.js");
const fetch_js_1 = require("./fetch.js");
const FetchError_js_1 = require("./FetchError.js");
const util_js_1 = require("./util.js");
const defaultEncoding = 'utf8';
const pipeline = (0, util_1.promisify)(Stream.pipeline);
async function readFile(filename, encoding) {
    const url = (0, util_js_1.toURL)(filename);
    if (!(0, util_js_1.isSupportedURL)(url)) {
        throw new Error('Unsupported network protocol');
    }
    return (0, util_js_1.isFileURL)(url) ? _readFile(url, encoding) : _fetchURL(url, encoding);
}
exports.readFile = readFile;
function _readFile(url, encoding) {
    // Convert it to a string because Yarn2 cannot handle URLs.
    const filename = (0, url_1.fileURLToPath)(url);
    return _read(() => fs.createReadStream(filename), (0, util_js_1.isZipped)(filename), encoding);
}
async function _fetchURL(url, encoding) {
    const response = await (0, fetch_js_1.fetch)(url);
    if (!response.ok) {
        throw FetchError_js_1.FetchUrlError.create(url, response.status);
    }
    return _read(() => response.body, (0, util_js_1.isZipped)(url), encoding);
}
async function _read(getStream, isZipped, encoding) {
    const stream = getStream();
    const decoder = (0, transformers_js_1.createDecoderTransformer)(encoding);
    const collector = createCollector(encoding || defaultEncoding);
    return isZipped ? pipeline(stream, zlib.createGunzip(), decoder, collector) : pipeline(stream, decoder, collector);
}
function readFileSync(filename, encoding) {
    const rawData = fs.readFileSync(filename);
    const data = (0, util_js_1.isZipped)(filename) ? zlib.gunzipSync(rawData) : rawData;
    return !encoding || encoding.startsWith('utf') ? (0, encode_decode_js_1.decode)(data) : data.toString(encoding);
}
exports.readFileSync = readFileSync;
function createCollector(encoding) {
    async function collect(iterable) {
        const buf = [];
        for await (const sb of iterable) {
            buf.push(typeof sb === 'string' ? sb : sb.toString(encoding));
        }
        return buf.join('');
    }
    return collect;
}
//# sourceMappingURL=fileReader.js.map